!    
!    LICENSE NOTICE
!
!    bdens: sums and densities of coupled anharmonic vibrations
!    Copyright (C) 2016 Collin G. L. Li, Thanh Lam Nguyen, and John R. Barker
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License: <http://www.gnu.org/licenses/>.
!
!-----------------------------------------------------------------------------------------
      REAL(8) FUNCTION feswitch( Emax,DELE,LTMODE,MVAL,KEYWORD)
      IMPLICIT NONE
      REAL(8), INTENT(IN) :: Emax, DELE, MVAL
      CHARACTER(6), INTENT(IN):: LTMODE, KEYWORD
      REAL(8) :: Time, Gx, C, Beta 
      Real(8) :: a1, alpha, b1, Be                  !Tadensum = b1*(Emax - x)^Be, Tnewloops = a1*Gx^alpha
      Real(8) :: Eswitch                                 ! MVAL = switch point energy if manually set
      REAL(8) :: dgxsolver, gxsolver
      EXTERNAL gxsolver, dgxsolver

!     Beta = ((ns-1)/ns)*(Sqmean/Meansq) 
      Time = MVAL
      Gx = sqrt((Time-0.830d0)*(1/9.350d0)*10.0d0**10)
      Beta = ((ns-1.0d0)/ns)*(Sqmean/Meansq)

!
!       G(X) SOLVER  --Practical limit
!
!     C = (s!w(s)*G)^(1/s) = (Sfac*Prod*Gx)^(1/ns) 

      IF ( LTMODE.EQ.'PRAC' ) THEN
        C = (Sfac*Prod)
!        C = DExp((1.0d0/ns)*DLOG(Sfac*Prod*Gx))      !Recursive Method
        feswitch = gxsolver( C, zpe, DELE, Beta, ns, Gx, Emax )    
        IF(feswitch .GT. Emax)THEN
	      feswitch = Emax
        ENDIF
        feswitch = INT( feswitch/DELE )*DELE
        write(*,99052) LTMODE, feswitch
!      ENDIF
!
!       dG(x) SOLVER  --Optimum limit
!
!     C = a1*alpha*ns/(b1*Be*(Sfac*Prod)^alpha)
!
      ELSEIF ( LTMODE.EQ.'AUTO' ) THEN
        a1 = 1.0d-5                                         !empirical results  1.616E-5 is the calculated result
        b1 = 0.07544d-00*ns-0.4317d-00                              !empirical results
        alpha = 0.9813d-00
        Be = 1.1d-00										
        C = a1*alpha*ns/(b1*Be*(Sfac*Prod)**alpha)

      write(*,*) a1, b1, alpha, Be, C

        feswitch = dgxsolver( Emax,C,zpe,Be,ns,alpha,b1,Beta,KEYWORD )
        feswitch = INT( feswitch/DELE )*DELE                ! makes eswitch an integer multiple of the grain size
        write(*,99052) LTMODE, feswitch

      ELSEIF (LTMODE.EQ.'MAN'.OR.LTMODE.EQ.'Man'.OR.LTMODE.EQ.'man')THEN
        feswitch = INT( MVAL/DELE )*DELE                    ! makes eswitch an integer multiple of the grain size
        write(*,99052) LTMODE, feswitch

      ELSE
        write(*,*) 'FATAL: unknown Eswitch KEYWORD: ', LTMODE
        write(3,*) 'FATAL: unknown Eswitch KEYWORD: ', LTMODE
        write(4,*) 'FATAL: unknown Eswitch KEYWORD: ', LTMODE
        write(5,*) 'FATAL: unknown Eswitch KEYWORD: ', LTMODE
        STOP
      ENDIF

      RETURN 
99052 FORMAT ( A6, ': Eswitch = ', F7.1 )
      END FUNCTION feswitch
      
