      SUBROUTINE BOOK1(Mol,E,Temp,IP,Time,Delt,Igo)
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c	Bookkeeping routine, works in concert with Subroutine STEPPER
c
c	Mol	= index of molecule at time=Time
c	E	= vibrational energy (relative to ZPE of Mol) at t=Time
c	Temp	= translational temperature
c	IP	= index of selected process:
c                 IP = rxn channel number for rxn
c			or IP = (Nchan+1) for collision
c	Time	= time
c	Delt	= time step
c    Ntime = max number of time steps
c
c	Igo	= flag which signals the following:
c		= 0	stop stepping for this trial
c		= 1	continue stepping for this trial
c
c***********************************************************************
c
c Date   Version  Revisions
c
c 08/02  v1.2.3  -Bug Fix: in subroutine book1.f, fixed bug that gave 
c                 erroneously large yields wen the time scale was much shorter
c                 than the inverse of the sum of the rates (k(E) + collision
c                 frequency).
c 05/02  v1.2.2  -Revised averaging of kuni so that only one sample per time bin per
c                 trial is included in the average. Previous algorithm resulted in
c                 over-weighting faster k(E)'s and gave erroneously high average kuni
c                 reported in multiwell.out.
c
c***********************************************************************

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
      INTEGER Mol , IP , Igo , ifin , i , Last , N
      INTEGER ifind , Dlast , k , istart 
     &      , ktime , kstep , klast , kfin
      DOUBLE PRECISION E , Temp , Time , Delt , COLSTEP , Step , rat
      DOUBLE PRECISION CC, Enew, DOUBNTERP
      SAVE 

      kstep = Ntime                             ! number of time steps for collision sampling

      N = 1 + E/DistStep                        ! Bin energy index for binned distributions
      IF ( N .GT. Ndist ) N = Ndist

c
c	First, record the current step
c
      IF ( Time.EQ.0.0D+00 ) THEN                      ! First step of a trial
         Last = 1                                      ! index of final bin for last step
         Dlast = 1                                     ! index of final bin for vib distribution
         klast = 1
         Num(Mol,1) = Num(Mol,1) + 1.d+00              ! Number of times 1st time bin accessed (for averages)
         Mum(Mol,1) = Mum(Mol,1) + 1.d+00              ! Number of times 1st time bin accessed (for EDIST)
         EMol(Mol,1) = EMol(Mol,1) + E                 ! For calculating average vibrational energy
         EDIST(Mol,N,1) = EDIST(Mol,N,1) + 1.d+00      ! For calculating vibrational distribution
         DO k = 1 , Nchan(Mol)                         ! Rab(k)= k(E) for kth rxn channel calculated by STEPPER [includes IVR effects]
            kuni(Mol,k,1) = Rab(k) + kuni(Mol,k,1)     ! for computing average rate constant [includes IVR effects]
         END DO  ! k
      ENDIF

      IF ( Tspec.EQ.'TIME' .OR. Tspec.EQ.'COLL') THEN
          ifin = 1 + ((Time+Delt)/Tlim)*(Ntime-1)          ! index of final time-bin for this step
        ELSEIF ( Tspec.EQ.'LOGT' ) THEN
          rat = log(( Time + Delt + 1.d-18 )/1.d-18 ) 
          rat = rat / log( Tlim/1.d-18)
          ifin = (Ntime)*rat
      ENDIF
      
c      write(*,*) Tspec, Tlim, Time, Delt, Ntime, ifin

      IF ( Time+Delt .GE. Tlim ) THEN
         ifin = Ntime
         Igo = 0                                       ! Stop stepping
      ENDIF

      IF ( ifin.GT.Last ) THEN                         ! snapshot at end of each time interval
         DO i = Last+1 , ifin                          ! ifin is LE Ntime (see earlier lines)
            Num(Mol,i) = Num(Mol,i) + 1.d+00           ! Number of times ith time bin accessed
            EMol(Mol,i) = EMol(Mol,i) + E              ! for average vibrational energy
            DO k = 1 , Nchan(Mol)                      ! Rab(k)= k(E) for kth rxn channel calculated by STEPPER [includes IVR effects]
               kuni(Mol,k,i) = Rab(k) + kuni(Mol,k,i)  ! for computing average rate constant [includes IVR effects]
            END DO ! k
         END DO ! i
         Last = ifin                                   ! re-set Last
      ENDIF

      ifind = 1 + (Time+Delt)*(Mtime-1)/Tlim           ! Vib distribution final time-bin index
      IF ( ifind.GT.Mtime ) ifind = Mtime
  
      IF ( ifind.GT.Dlast ) THEN                       ! snapshot at end of each time interval
         DO i = Dlast + 1 , Mtime                      ! ifind is LE Mtime (see earlier lines)
           Mum(Mol,i) = Mum(Mol,i) + 1.d+00            ! Number of times ith time bin accessed
           EDIST(Mol,N,i) = EDIST(Mol,N,i) + 1.d+00    ! Energy-dependent vibrational distribution (N is index of energy)
         END DO ! i
         Dlast = ifind                                 ! re-set Dlast
      ENDIF


c--------This block for sampling molecular energies    <------------------------- FOR ENERGIES
c      *** Normally, commented-out ***
c         kfin = 1 + (Time+Delt)*(kstep-1)/Tlim          ! time-step index for collisional sampling
c         IF ( kfin.GT.klast ) THEN                      ! snapshot at end of each time interval
c            DO i = klast + 1 , MIN(kfin,kstep)
c               WRITE (20,*) E
c            END DO
c            klast = kfin
c         ENDIF
c------End energy sampling block-------------

c
c	Now apply the stochastic changes
c 
      Time = Time + Delt
 
      IF ( Igo.EQ.0 ) THEN                              ! Stop stepping
         RETURN
      ELSEIF ( IP .EQ. (1+Nchan(Mol)) ) THEN            ! COLLISION
         Step = COLSTEP(Mol,E,Temp)                     ! Collision step size
         E = E + Step                                   ! Energy after collision
c         write(21,*) Step   !         <----------------un-comment this line FOR WRITING COLLISION STEP-SIZES (in Unit 21)
      ELSEIF ( Jto(Mol,IP) .LE. Nwells ) THEN
         Enew = E + HMol(Mol) - HMol(Jto(Mol,IP))
         Mol = Jto(Mol,IP)                             ! New WELL index
         E = Enew                                      ! Energy of new Well
      ELSEIF ( Jto(Mol,IP) .GT. Nwells ) THEN
         Mol = Jto(Mol,IP)                             ! New PRODUCT index
      ENDIF
 
      IF ( Mol.GT.NWells .AND. Last.LT.Ntime ) THEN     ! If irreversible product
         Igo = 0                                        ! Stop stepping
         DO  i = Last + 1 , Ntime
            Num(Mol,i) = Num(Mol,i) + 1.d+00            ! Number of times ith bin accessed
         END DO
      ENDIF
 
      RETURN
      END
