c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2011 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

      SUBROUTINE QKinf(Mol,nchann,tx,Kinf,Ainf,Einf,kcol,kosc)

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol , nchann , i , IE , Itop , mm, i2 , istart
      DOUBLE PRECISION Q , RK , TT , E , D , DENVR , Kinf , Ainf , 
     &                 Einf , K , tx , Et , arg , Y1 , Y2 , Y3 , Y , 
     &                 z , Estep, RR, QQ , Qlast , Rlast , TTSTEP , 
     &                 TS , EC , x , arg2 , rat , f , tcor , aa, bb
      DOUBLE PRECISION kcol, Qts, QQts, Qtslast, DENTS, E2, Qcheck
      DOUBLE PRECISION kosc(MaxWell,MaxChan) , RVR
      DIMENSION K(3), TS(3)
      SAVE 
c
c	Calculates high pressure limiting unimolecular rate constant 
c         (See user Manual for details)
c
c	Mol	= index of molecule
c	nchann	= index of rxn channel
c	tx	= translational temperature
c	Kinf	= infinite pressure rate constant [Robinson & Holbrook Eq. 4.30]
c	Ainf	= A-factor for kinf
c	Einf	= Activation energy for kinf
c	RVR     = interpolated value of k(E); see subroutine RVR.f
c
c
      TTSTEP = 0.01d+00/tx                       ! Step size in 1/Temp; tx = input temperature
      TT = 1.0d+00/tx-2.d+00*TTSTEP               ! 1/Temp (starting value)
       
      DO mm = 1,3
        K(mm) = 0.0d+00
        RK = 0.0d+00
        Q  = 0.0d+00
        Qts = 0.0d+00
        Qlast = 0.0d+00
        Qtslast = 0.0d+00
        Rlast = 0.0d+00

        TT = TT + TTSTEP                           ! 1/T
        TS(mm) = TT                                ! 1/T

        IF ( NCENT(Mol,nchann) .NE. -2 ) THEN
          EC = Eor(Mol,nchann)                     ! NORMAL METHODS include centrifugal correction
        ELSE
          EC = Eo(Mol,nchann)                      ! LEGACY METHOD DOES NOT include centrifugal correction
        ENDIF
      
        Estep = Egrain1
        E = 0.0d+00                                ! active energy in reactant, or TS
        arg = EC                                   ! active energy in excited reactant
        DO i = 1 , imax1                           ! summation continues until end of first part of double array
          QQ = DENVR( Mol,E )*exp(-E*hor*TT)           ! molecule partition fxn; E=active energy in molecule; density stored as log
          Q  = Q  + 0.5*Estep*(QQ + Qlast)             ! molecule partition fxn; Trapezoidal Rule
          Qlast = QQ
          D = DENVR(Mol,arg)                          ! density of states of EXCITED molecule ( Eor = Eo for NOCENT )
          QQts = D*exp(-arg*hor*TT)                    ! partition fxn Q*
          RR = D*RVR( Mol,nchann,E )*exp(-arg*hor*TT)  ! thermal rate constant; arg=active energy in molecule; RVR = interpolated k(E)
          RK = RK + 0.5*Estep*(RR + Rlast)             !  +++++++ rate constant k(inf) ++++++++++++
          Qts = Qts + 0.5*Estep*(QQts+Qtslast)         ! partition fxn for use in strong collider calculation
          Rlast = RR
          Qtslast = QQts
          E = E + Estep                                ! E = active energy in thermal reactant
          arg = EC + E                                 ! arg = active energy in TS ( >EC ) 
        END DO  ! i
c
c       NOW CONTINUE TO HIGHER ENERGIES
c 
        Estep = 0.05*tx/hor                           ! New step size (cm-1)
        IF ( Estep. LT. Egrain1 ) Estep = Egrain1
        DO WHILE ( arg.LT.(Emax2-Estep) )            ! summation continues for second part of double array
          QQ = DENVR( Mol,E )*exp(-E*hor*TT)           ! molecule partition fxn; E=active energy in molecule; density stored as log
          Q  = Q  + 0.5*Estep*(QQ + Qlast)             ! molecule partition fxn; Trapezoidal Rule
          Qlast = QQ
          D = DENVR(Mol,arg)                           ! density of states of EXCITED molecule ( Eor = Eo for NOCENT )
          QQts = D*exp(-arg*hor*TT)                    ! partition fxn Q*
          RR = D*RVR( Mol,nchann,E )*exp(-arg*hor*TT)  ! thermal rate constant; arg=active energy in molecule; k(E) stored as log
          RK = RK + 0.5*Estep*(RR + Rlast)             !  +++++++ rate constant k(inf) ++++++++++++
          Qts = Qts + 0.5*Estep*(QQts+Qtslast)         ! partition fxn for use in strong collider calculation
          Rlast = RR
          Qtslast = QQts
          E = E + Estep
          arg = EC + E                            ! arg=active energy in TS ( >EC )
        END DO  ! i

       K(mm) = RK/Q                               ! Rate constant, including any centrifugal corrections
                    
       IF ( mm .EQ. 2) THEN
         kosc(Mol,nchann) = kcol*Qts/Q            ! Strong-collider low pressure limit rate constant
       ENDIF

      END DO  ! m=1,3 for T-dependence
c
c     Now find Arrhenius parameters
c
      Kinf = K(2)
      IF ( K(1)/K(3) .GT. 0.d+00 ) THEN 
        Et = -LOG( K(1)/K(3) )/( TS(1) - TS(3) )              ! Et = Ea/R
        Einf = (1.9872065d-03)*Et                             ! kcal/mole
        Ainf = K(2)*exp( Et*TS(2) )                           ! 1/s
      ELSE
        Einf = 0.0
        Ainf = 0.0
      ENDIF
      
      IF ( NCENT(Mol,nchann) .EQ. -2 ) THEN      ! Legacy centrifugal correction (not recommended)
        IF (    (MolMom(Mol) .GT. 0.0 )            
     &   .AND. (TSmom(Mol,nchann) .GT. 0.0)  )  THEN
         rat = TSmom(Mol,nchann)/MolMom(Mol)
        ELSE
         rat = 1.0d+00
        ENDIF
        Kinf = rat*Kinf
        Ainf = rat*Ainf
        kosc(Mol,nchann) = rat*kosc(Mol,nchann)
      ENDIF
      
      RETURN
      END
