!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!    
!    LICENSE NOTICE
!
!    Copyright (C) 2019 Michele Ceotto, Chiara Aieta, Fabio Gabas, 
!                       Thanh Lam Nguyen, and John R. Barker
!
!    Contact:
!
!    Michele Ceotto  (email: michele.ceotto@unimi.it)
!    Dipartimento di Chimica
!    Università degli Studi di Milano
!    via Golgi 19, Milano 20133, ITALY
!
!    or:
!
!    John R. Barker   (email: jrbarker@umich.edu)
!    Department of Atmospheric, Oceanic, and Space Sciences
!    College of Engineering
!    University of Michigan
!    Ann Arbor, MI 48109
!
!    http://clasp-research.engin.umich.edu/multiwell
!    This program is free software; you can redistribute it and/or
!    modify it under the terms of the GNU General Public License (version 2)
!    as published by the Free Software Foundation.
!   
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
!    GNU General Public License for more details.
!   
!    See the 'ReadMe' file for a copy of the GNU General Public License,
!    or contact:
!   
!    Free Software Foundation, Inc.
!    59 Temple Place - Suite 330
!    Boston, MA 02111-1307, USA.
!
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!                                                                 
!                         PROGRAM paradensum    
!
!                               by      
!
!           by Michele Ceotto, Chiara Aieta, Fabio Gabas,
!               Thanh Lam Nguyen, and John R. Barker    
!                                                                  
!           ***PARallel Anharmonic DENsities and SUM of states***
!
!                             based on
!
!          The parallel implementation* of the Wang-Landau algorithms 
!          for densities of states**
!
!    Literature Citations:                                                    
!    *parallel implementation of anaharmonic density of states
!    C. Aieta, F. Gabas and M. Ceotto, J. Phys. Chem. A., 120(27), 
!    4853-4862 (2016).
!                                                                  
!    **Density of states algorithms
!    F. Wang and D. P. Landau, Phys. Rev. Letters 86, 2050-2053 (2001).           
!    M. Basire, P. Parneix, and F. Calvo, J. Chem.Phys. 129, 081101 (2008).    
!    T. L. Nguyen and J.R. Barker, J. Phys. Chem. A., 114, 3718–3730 (2010).
!    
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

!        SUBROUTINE SHRLEV(Ttot,AT,DELE,JMAX,B,V,n1,n2,IMAX,zpe,
!     &          Vhr,Bhr,Phav,Phab,Nsig)
        SUBROUTINE SHRLEV(DELE,JMAX,B,V,n1,n2,IMAX_1,zpe_1,
     &          Vhr,Bhr,Phav_1,Phab_1,Nsig_1)

        USE decl_alloc 

        IMPLICIT REAL(8)(A-H,O-Z), INTEGER(4)(I-N)
        PARAMETER (NN=2000, No=1)
        !DIMENSION EV(NN), CV(No), CF(No), T(JMAX), AT(JMAX)
        DIMENSION EV_2(NN), CV(No), CF(No) 
         DIMENSION IR(NN)
        CHARACTER Vhr*5, Bhr*5

        Emax=(JMAX-1)*DELE

        NCV=1
        NCF=1
        CV(1)=V
        CF(1)=B

        CALL GHRLEV(EV_2,NN,IMAX,Emax,B,NCV,NCF,CV,CF,n1,n2,
     &          Vhr,Bhr,Phav,Phab)

      zpe_1=EV_2(1)

C......      write(*,*) "Zero-point energy = ", zpe

      DO I=1, IMAX_1
            tp=EV_2(I)-zpe_1
            IR(I)=NINT(tp/DELE)
C......            write(*,*) I, tp

      ENDDO 

      DO J=1, IMAX_1
            LL=IR(J)
            DO L=1, JMAX - LL
                  AT(L+LL) = AT(L+LL) + Ttot(L)
            ENDDO
      ENDDO
      DO J=1, JMAX
            Ttot(J)=AT(J)/Nsig_1
            AT(J)=0.0d0
      ENDDO 

      RETURN
      END


